Enum TxVersion
pub enum TxVersion {
Sprout(u32),
V3,
V4,
V5,
V6,
}Expand description
The set of defined transaction format versions.
This is serialized in the first four or eight bytes of the transaction format, and
represents valid combinations of the (overwintered, version, version_group_id)
transaction fields. Note that this is not dependent on epoch, only on transaction encoding.
For example, if a particular epoch defines a new transaction version but also allows the
previous version, then only the new version would be added to this enum.
Variants
Sprout(u32)
Transaction versions allowed prior to Overwinter activation. The argument MUST be
in the range 1..=0x7FFFFFFF. Only versions 1 and 2 are defined; 3..=0x7FFFFFFF
was allowed by consensus but considered equivalent to 2. This is specified in
 7.1 Transaction Encoding and Consensus.
V3
Transaction version 3, which was introduced by the Overwinter network upgrade and allowed until Sapling activation. It is specified in  7.1 Transaction Encoding and Consensus.
V4
Transaction version 4, which was introduced by the Sapling network upgrade. It is specified in  7.1 Transaction Encoding and Consensus.
V5
Transaction version 5, which was introduced by the NU5 network upgrade. It is specified in  7.1 Transaction Encoding and Consensus and ZIP 225.
V6
Transaction version 6, specified in ZIP 229.
Implementations
impl TxVersion
impl TxVersion
pub fn read<R>(reader: R) -> Result<TxVersion, Error>where
R: Read,
pub fn header(&self) -> u32
pub fn version_group_id(&self) -> u32
pub fn write<W>(&self, writer: W) -> Result<(), Error>where
W: Write,
pub fn has_sprout(&self) -> bool
pub fn has_sprout(&self) -> bool
Returns true if this transaction version supports the Sprout protocol.
pub fn has_overwinter(&self) -> bool
pub fn has_sapling(&self) -> bool
pub fn has_sapling(&self) -> bool
Returns true if this transaction version supports the Sapling protocol.
pub fn has_orchard(&self) -> bool
pub fn has_orchard(&self) -> bool
Returns true if this transaction version supports the Orchard protocol.
pub fn has_ironwood(&self) -> bool
pub fn has_ironwood(&self) -> bool
Returns true if this transaction version supports the Ironwood protocol.
pub fn suggested_for_branch(consensus_branch_id: BranchId) -> TxVersion
pub fn suggested_for_branch(consensus_branch_id: BranchId) -> TxVersion
Suggests the transaction version that should be used in the given Zcash epoch.
pub fn valid_in_branch(&self, consensus_branch_id: BranchId) -> bool
pub fn valid_in_branch(&self, consensus_branch_id: BranchId) -> bool
Returns true if this transaction version is valid for us in the specified consensus
branch, false otherwise.